// Author:	Sunrom Technologies  6/3/2012
// Board: Ultrasonic Sensor PWM Out Demo  Model: 3719  at Sunrom.com
// The program can Measure distance and output as serial string
// Compiler: Keil C51        MCU: AT89S52

#include <reg52.h>		// standard 8051 defines
#include <stdio.h>		// for printf()
#include<intrins.h> 	// for nop()

// Sensor Connections
sbit TRIG = P3^3;	
// Note: ECHO pin is connected to P3_2 which is also INT0 interrupt pin, this cannot be changed

unsigned int T0_ISR_count = 0;

unsigned long high_time_us;	// 32 bit variable, stores pulse time in uS
unsigned long distance_cm;	// 32 bit variable, Distance Value in Centimeter
unsigned long distance_in;	// 32 bit variable, Distance Value in Inches

unsigned int i;

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=  Timer 0 Overflow Interrupt  -=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void T0_ISR (void) interrupt 1
{
	T0_ISR_count++;					// Increment overflow counter
	TF0 = 0;                        // Reset the interrupt request
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=-=-=-  Delay x ms       -=-=-=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void delay_ms(int x)	 // delays x msec (at fosc=11.0592MHz)
{
	int y=0;
	while(x>=0)
	{
		for (y=0; y<100; y++);
		x--;
	}
}
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=-=-=-  Delay 10uS       -=-=-=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#define delay_10us() _nop_();_nop_();_nop_();_nop_(); _nop_();_nop_();_nop_();_nop_()

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=-=-=-  Init Serial Port -=-=-=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void serial_init() {
	/*--------------------------------------
	Set serial port for 9600 baud at
	11.0592 MHz.  Note that we use Timer 1
	for the baud rate generator.
	--------------------------------------*/
	SCON  = 0x50;
	TMOD |= 0x20;
	TH1   = 0xFA;
	TR1   = 1;
	TI    = 1;
	PCON |= 0x80;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// -=-=-=-=-  Main Program     -=-=-=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void main (void)
{

	serial_init(); // Init serial port at 9600

	TRIG = 0;  // Keep pin low at startup

	printf ("\nUltrasonic Distance Sensor\n");

	/*--------------------------------------
	Enable interrupts for timer 0.
	--------------------------------------*/
	ET0 = 1;
	EA = 1;
	
	/*--------------------------------------
	Set Timer0 for 16-bit interval timer
	mode.
	--------------------------------------*/
	TMOD = (TMOD & 0xF0) | 0x09;
	
	while (1)   // Loop Forever
	  {

// Send 10uS trigger Pulse
		 TRIG = 1;		// Trigger Pin Made High
		 delay_10us(); 	// Wait 10uS
		 TRIG = 0;		// Make it low, this starts ranging process


// Start pulse measurement
	/*--------------------------------------
	Clear the timer overflow counter and
	the timer low and high registers.  Then,
	start the timer.
	--------------------------------------*/

	  T0_ISR_count = 0;	// Timer0 Counter made zero
	  TH0 = 0;
	  TL0 = 0;
	
	  TR0 = 1;			// Start Timer0
	
	/*--------------------------------------
	Wait for the pulse to start. Then, wait for the pulse to end.
	--------------------------------------*/
	  i=0;
	  while (!INT0)
	  {
         i++;
         if(i>20000) // timeout, so abort
            break;
      }
	  i=0;
	  while (INT0)
	  {
         i++;
         if(i>20000) // timeout, so abort
            break;
      }
	
	/*--------------------------------------
	Compute the width of the pulse -- one
	clock cycle is 1.085us for a standard 8051 at 11.0592 Mhz Crystal
	--------------------------------------*/

	  high_time_us = (unsigned long)((TH0 << 8) | TL0 | ((unsigned long)T0_ISR_count << 16))*1.085; 

	  distance_cm = high_time_us/58;		// calculate Distance in Centimeters
	  distance_in = high_time_us/148;		// calculate Distance in Inches

		// Print the values to serial port
	   if(distance_cm > 0 && distance_cm < 500)  // check valid range then only print
	   	printf("Distance: %6lu cm  %6lu Inch  PW=%6lu uS\n", distance_cm, distance_in, high_time_us);

		delay_ms(250);
	  }
}

/*---------------------------------------------------------
---------------------------------------------------------*/

